# -*- coding: utf-8 -*-
from __future__ import division

from numpy import (maximum as max_, logical_not as not_, absolute as abs_, minimum as min_, select, where)

from openfisca_france.model.base import *  # noqa analysis:ignore

class parisien(Variable):
    value_type = bool
    entity = Famille
    definition_period = MONTH
    label = u"Résident à Paris au moins 3 ans dans les 5 dernières années"

class paris_base_ressources_i(Variable):
    value_type = float
    label = u"Base de ressources pour un individu, pour l'ensemble des aides de Paris"
    entity = Individu
    definition_period = MONTH

    def formula(individu, period, legislation):
        last_year = period.last_year

        ass = individu('ass', period)
        aah = individu('aah', period)
        asi = individu('asi', period)
        caah = individu('caah', period)

        salaire_net = individu('salaire_net', period)
        indemnites_stage = individu('indemnites_stage', period)
        smic = legislation(period).paris.smic_net_mensuel
        indemnites_stage_imposable = where((smic >= indemnites_stage), indemnites_stage, 0)
        revenus_stage_formation_pro = individu('revenus_stage_formation_pro', period)

        chomage_net = individu('chomage_net', period)
        allocation_securisation_professionnelle = individu('allocation_securisation_professionnelle', period)
        indemnites_journalieres = individu('indemnites_journalieres', period)
        indemnites_chomage_partiel = individu('indemnites_chomage_partiel', period)
        indemnites_volontariat = individu('indemnites_volontariat', period)

        prestation_compensatoire = individu('prestation_compensatoire', period)
        retraite_nette = individu('retraite_nette', period)
        pensions_invalidite = individu('pensions_invalidite', period)

        def revenus_tns():
            revenus_auto_entrepreneur = individu('rpns_auto_entrepreneur_benefice', period, options = [ADD])

            # Les revenus TNS hors AE sont estimés en se basant sur le revenu N-1
            tns_micro_entreprise_benefice = individu('rpns_micro_entreprise_benefice', last_year) / 12
            tns_benefice_exploitant_agricole = individu('rpns_benefice_exploitant_agricole', last_year) / 12
            tns_autres_revenus = individu('rpns_autres_revenus', last_year) / 12

            return revenus_auto_entrepreneur + tns_micro_entreprise_benefice + tns_benefice_exploitant_agricole + tns_autres_revenus

        result = (
            ass + aah + asi + caah
            + salaire_net + indemnites_stage_imposable + revenus_stage_formation_pro
            + chomage_net + allocation_securisation_professionnelle + indemnites_journalieres + indemnites_chomage_partiel + indemnites_volontariat
            + prestation_compensatoire + retraite_nette + pensions_invalidite
            + revenus_tns()
            )

        return result

class paris_base_ressources_famille(Variable):
    value_type = float
    label = u"Base de ressources liée à une famille (à ajouter aux ressources des individus), pour l'ensemble des aides de Paris"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):
        aeeh = famille('aeeh', period)
        aspa = famille('aspa', period)
        rsa = famille('rsa', period)
        return aeeh + aspa + rsa

class paris_base_ressources_couple(Variable):
    value_type = float
    label = u"Base de ressources pour un couple, pour l'ensemble des aides de Paris"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):
        en_couple = famille('en_couple', period)
        ressources_demandeur = famille.demandeur('paris_base_ressources_i', period)
        ressources_conjoint = famille.conjoint('paris_base_ressources_i', period)
        ressources_famille = famille('paris_base_ressources_famille', period)

        return where(en_couple,
            ressources_demandeur + ressources_conjoint + ressources_famille,
            ressources_demandeur + ressources_famille)

class paris_base_ressources_foyer(Variable):
    value_type = float
    label = u"Base de ressources pour un foyer, pour l'ensemble des aides de Paris"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):
        ressources = famille.members('paris_base_ressources_i', period)
        ressources_famille = famille('paris_base_ressources_famille', period)

        return famille.sum(ressources) + ressources_famille

class paris_indemnite_enfant_i(Variable):
    value_type = float
    label = u"Indemnités de maternité, paternité, adoption"
    entity = Individu
    definition_period = MONTH

    def formula(individu, period):

        indemnites_maternite = individu('indemnites_journalieres_maternite', period)
        indemnites_paternite = individu('indemnites_journalieres_paternite', period)
        indemnites_adoption = individu('indemnites_journalieres_adoption', period)

        result = indemnites_maternite + indemnites_paternite + indemnites_adoption

        return result

class paris_indemnite_enfant(Variable):
    value_type = float
    label = u"Base de ressources pour Indemnités de maternité, paternité, adoption"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):

        paris_indemnite_enfant_i = famille.members('paris_indemnite_enfant_i', period)
        paris_indemnite_enfant = famille.sum(paris_indemnite_enfant_i)

        return paris_indemnite_enfant

class paris_enfant_handicape(Variable):
    value_type = bool
    label = u"Enfant handicapé au sens de la mairie de Paris"
    entity = Individu
    definition_period = MONTH

    def formula(individu, period):

        handicap = individu('handicap', period)
        paris_enfant = individu('paris_enfant', period)

        return paris_enfant * handicap

class paris_enfant(Variable):
    value_type = bool
    label = u"Enfant pris en compte par la mairie de Paris"
    entity = Individu
    definition_period = MONTH

    def formula(individu, period):

        est_enfant_dans_famille = individu('est_enfant_dans_famille', period)
        enfant_place = individu('enfant_place', period)
        a_charge_fiscale = individu('enfant_a_charge', period.this_year)

        return est_enfant_dans_famille * (1 - enfant_place) * a_charge_fiscale

class paris_enfant_garde_alternee(Variable):
    value_type = bool
    label = u"Enfant en garde alternée pris en compte par la mairie de Paris"
    entity = Individu
    definition_period = MONTH

    def formula(individu, period):

        garde_alternee = individu('garde_alternee', period)
        paris_enfant = individu('paris_enfant', period)

        return garde_alternee * paris_enfant

class paris_enfant_handicape_garde_alternee(Variable):
    value_type = bool
    label = u"Enfant handicapé en garde alternée pris en compte par la mairie de Paris"
    entity = Individu
    definition_period = MONTH

    def formula(individu, period):

        garde_alternee = individu('garde_alternee', period)
        paris_enfant_handicape = individu('paris_enfant_handicape', period)

        return garde_alternee * paris_enfant_handicape


class paris_nb_enfants(Variable):
    value_type = float
    label = u"Nombre d'enfant dans la famille"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):

        nb_enfants = famille.members('paris_enfant', period)
        paris_nb_enfants = famille.sum(nb_enfants)

        return paris_nb_enfants


class paris_nb_enfants_handicapes(Variable):
    value_type = float
    label = u"Nombre d'enfants handicapés dans la famille"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):
        nb_enfants = famille.members('paris_enfant_handicape', period)
        return famille.sum(nb_enfants)


class paris_taux_effort(Variable):
    value_type = float
    label = u"Taux d'effort pour le loyer"
    entity = Famille
    definition_period = MONTH
    reference = "Articles II.2.1.b.3 et III.2.1.b.3 et IV.2.3.b.3 et V.3.1.b.3 du Règlement Municipal"

    def formula(famille, period, legislation):

        loyer = famille.demandeur.menage('loyer', period)
        charges_forfaitaires_logement = famille('aide_logement_charges', period)
        aide_logement = famille('aide_logement', period)

        base_ressources = famille('paris_base_ressources_foyer', period.last_month)
        aspa_reference = famille('paris_aspa_reference', period)
        ressources = max_(aspa_reference, base_ressources)

        return (loyer + charges_forfaitaires_logement - aide_logement) / ressources


class paris_condition_taux_effort(Variable):
    value_type = bool
    label = u"Condition vérifiant si le taux d'effort est suffisamment élevé"
    entity = Famille
    definition_period = MONTH
    reference = "Articles II.2.1.b.3 et III.2.1.b.3 et IV.2.3.b.3 et V.3.1.b.3 du Règlement Municipal"

    def formula(famille, period, legislation):

        taux_effort = famille('paris_taux_effort', period)
        taux_effort_min = legislation(period).paris.paris_logement.taux_effort_min

        return taux_effort >= taux_effort_min


class paris_locataire(Variable):
    value_type = bool
    label = u"Famille qui est locataire de son logement"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):

        statut_occupation_logement = famille.demandeur.menage('statut_occupation_logement', period)

        return (
            (statut_occupation_logement == TypesStatutOccupationLogement.locataire_hlm) +
            (statut_occupation_logement == TypesStatutOccupationLogement.locataire_vide) +
            (statut_occupation_logement == TypesStatutOccupationLogement.locataire_meuble) +
            (statut_occupation_logement == TypesStatutOccupationLogement.locataire_foyer)
            )

class paris_logement_a_charge(Variable):
    value_type = bool
    label = u"Famille qui acquitte ses charges de logement"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):

        statut_occupation_logement = famille.demandeur.menage('statut_occupation_logement', period)
        locataire = famille('paris_locataire', period)

        return (
            locataire +
            (statut_occupation_logement == TypesStatutOccupationLogement.primo_accedant) +
            (statut_occupation_logement == TypesStatutOccupationLogement.proprietaire)
            )



# -*- coding: utf-8 -*-
from __future__ import division

from numpy import (maximum as max_, logical_not as not_, absolute as abs_, minimum as min_, select, where)
from math import ceil

from openfisca_france.model.base import *  # noqa analysis:ignore


class paris_logement_pa_plafond(Variable):
	value_type = float
	entity = Famille
	definition_period = MONTH
	label = u"Plafond de ressources pour l'éligibilité à l'aide Paris Logement pour les personnes âgées"
	reference = "article II.2.1.b/2 du règlement municipal du CASVP"

	def formula(famille, period, parameters):
		salaire_minimum = parameters(period).marche_travail.salaire_minimum.smic
		smic_brut_mensuel = salaire_minimum.smic_b_horaire * salaire_minimum.nb_heures_travail_mensuel

		# Utilisation des valeurs indicatives de service-public.fr pour passer du SMIC brut au SMIC net
		# https://www.service-public.fr/particuliers/vosdroits/F2300
		# Dans l'attente de la formule effectivement utilisée par la ville de Paris
		smic_net_mensuel = 7.94 / 10.03 * smic_brut_mensuel
		smic_arrondi = ceil(smic_net_mensuel / 10) * 10	# arrondi à la dizaine supérieure

		nb_enfants = famille('paris_nb_enfants', period)

		plafond_avec_enfant = parameters(period).paris.personnes_agees.paris_logement.plafond.avec_enfant

		return where(nb_enfants < 1, smic_arrondi, plafond_avec_enfant)


class paris_logement_pa_eligibilite(Variable):
	value_type = bool
	entity = Famille
	definition_period = MONTH
	label = u"Éligibilité à l'aide Paris Logement pour les personnes âgées"
	reference = "article II.2.1.b du règlement municipal du CASVP"

	def formula(famille, period, parameters):
		pa_ph_eligibilite = famille('paris_logement_pa_ph_eligibilite', period)

		personnes_agees_i = famille.members('paris_personne_agee', period)
		personnes_agees = famille.any(personnes_agees_i)

		base_ressources = famille('paris_base_ressources_foyer', period.last_month)
		plafond = famille('paris_logement_pa_plafond', period)

		return pa_ph_eligibilite * personnes_agees * (base_ressources <= plafond)


class paris_logement_pa_montant(Variable):
	value_type = float
	entity = Famille
	definition_period = MONTH
	label = u"Montant de l'aide Paris Logement pour les personnes âgées"
	reference = "article II.2.1.a du règlement municipal du CASVP"

	def formula(famille, period, parameters):
		en_couple = famille('en_couple', period)
		nb_enfants = famille('paris_nb_enfants', period)
		param_montant = parameters(period).paris.personnes_agees.paris_logement.montant

		montant = select(
				[not_(en_couple), nb_enfants < 1, nb_enfants >=1],
				[param_montant.personne_isolee, param_montant.couple_sans_enfant, param_montant.couple_avec_enfant]
			)
		charge_logement = famille('paris_logement_charge_nette_mensuelle', period)

		return min_(montant, charge_logement)


class paris_logement_pa(Variable):
	value_type = float
	entity = Famille
	definition_period = MONTH
	label = u"Paris Logement pour les personnes âgées"
	reference = "article II.2.1 du règlement municipal du CASVP"

	def formula(famille, period, parameters):
		eligibilite = famille('paris_logement_pa_eligibilite', period)
		montant = famille('paris_logement_pa_montant', period)
		return eligibilite * montant



# -*- coding: utf-8 -*-
from __future__ import division

from numpy import (maximum as max_, logical_not as not_, absolute as abs_, minimum as min_, select, where)

from openfisca_france.model.base import *  # noqa analysis:ignore

# Paris solidarité pour les personnes agées et les personnes handicapées

class paris_logement_psol(Variable):
    value_type = float
    label = u"Montant de l'aide Paris Solidarité"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):

        parisien = famille('parisien', period)

        psol_pa = famille('paris_solidarite_pa', period)
        psol_ph = famille('paris_solidarite_ph', period)

        result = parisien * max_(psol_pa, psol_ph)

        return result



# -*- coding: utf-8 -*-
from __future__ import division

from numpy import (maximum as max_, logical_not as not_, absolute as abs_, minimum as min_, select, where)

from openfisca_france.model.base import *  # noqa analysis:ignore


class paris_complement_sante_ph_eligibilite(Variable):
	value_type = bool
	entity = Famille
	definition_period = MONTH
	label = u"Éligibilité au Complément Santé Paris pour les personnes handicapées"
	reference = "Article III.1.2.b.3 du règlement municipal du CASVP"

	def formula(famille, period, parameters):
		
		personne_handicapee = famille.members('paris_personne_handicapee', period)
		personnes_handicapees = famille.any(personne_handicapee)

		base_ressources = famille('paris_base_ressources_couple', period.last_month)

		param_plafond = parameters(period).paris.personnes_handicapees.paris_complement_sante.plafond
		en_couple = famille('en_couple', period)
		plafond_1 = where(en_couple, param_plafond.en_couple, param_plafond.personne_isolee)

		param_aah = parameters(period).prestations_sociales.prestations_etat_de_sante.invalidite
		plafond_2 = param_aah.aah.montant + param_aah.caah.majoration_vie_autonome

		return personnes_handicapees * (base_ressources <= max_(plafond_1, plafond_2)) 


class paris_complement_sante_ph_montant(Variable):
	value_type = float
	entity = Famille
	definition_period = MONTH
	label = u"Montant du Complément Santé Paris pour les personnes handicapées"
	reference = "Articles III.1.2.a.2 & III.1.2.b.5 du règlement municipal du CASVP"

	def formula(famille, period, parameters):
		
		montant = parameters(period).paris.personnes_handicapees.paris_complement_sante.montant
		acs = famille('acs', period)
		cmu_c = famille('cmu_c', period)

		return not_(cmu_c) * max_(montant - acs, 0)


class paris_complement_sante_ph(Variable):
	value_type = float
	entity = Famille
	definition_period = MONTH
	label = u"Complément Santé Paris pour les personnes handicapées"
	reference = "Article III.1.2 du règlement municipal du CASVP"

	def formula(famille, period, parameters):

		eligibilite = famille('paris_complement_sante_ph_eligibilite', period)
		montant = famille('paris_complement_sante_ph_montant', period)
		
		return eligibilite * montant



# -*- coding: utf-8 -*-
from __future__ import division

from numpy import (maximum as max_, logical_not as not_, absolute as abs_, minimum as min_, select, where)

from openfisca_france.model.base import *  # noqa analysis:ignore


class paris_solidarite_pa_base_ressources(Variable):
    value_type = float
    entity = Famille
    definition_period = MONTH
    label = u"Base ressources mensuelles pour Paris Solidarité pour les personnes âgées"
    reference = "article II.1.1.b.4 du règlement municipal du CASVP"

    def formula(famille, period, parameters):
        
        base_ressources = famille('paris_base_ressources_couple', period)
        aspa_reference = famille('paris_aspa_reference', period)

        return max_(aspa_reference, base_ressources)


class paris_solidarite_pa_montant(Variable):
    value_type = float
    entity = Famille
    definition_period = MONTH
    label = u"Montant mensuel pour Paris Solidarité pour les personnes âgées"
    reference = "article II.1.1.b.3 du règlement municipal du CASVP"

    def formula(famille, period, parameters):

        base_ressources = famille('paris_solidarite_pa_base_ressources', period)
        plafond_psol = parameters(period).paris.personnes_agees.psol.plafond

        en_couple = famille('en_couple', period)
        plafond = (en_couple * plafond_psol.couple + not_(en_couple) * plafond_psol.personne_seule)

        return max_(0, plafond - base_ressources)


class paris_solidarite_pa_eligibilite(Variable):
    value_type = bool
    entity = Famille
    definition_period = MONTH
    label = u"Éligibilité à Paris Solidarité pour les personnes âgées"
    
    def formula(famille, period, parameters):

        personnes_agees_i = famille.members('paris_personne_agee', period)
        return famille.any(personnes_agees_i)


class paris_solidarite_pa(Variable):
    value_type = float
    entity = Famille
    definition_period = MONTH
    label = u"Paris Solidarité pour les personnes âgées"
    
    def formula(famille, period, parameters):

        montant = famille('paris_solidarite_pa_montant', period)
        pa_eligibilite = famille('paris_solidarite_pa_eligibilite', period)
        
        return pa_eligibilite * montant



# -*- coding: utf-8 -*-
from __future__ import division

from numpy import (maximum as max_, logical_not as not_, absolute as abs_, minimum as min_, select, where)

from openfisca_france.model.base import *  # noqa analysis:ignore


class paris_personne_agee(Variable):
    value_type = bool
    label = u"Personne âgée"
    entity = Individu
    definition_period = MONTH

    def formula(individu, period, legislation):

        param_age = legislation(period).paris.personnes_agees.personnes_agees

        age = individu('age', period)
        inapte = individu('inapte_travail', period)

        aspa_eligibilite = individu('aspa_eligibilite', period)

        return (age >= param_age.age_min) + (inapte * (age >= param_age.age_min_si_inaptitude)) + aspa_eligibilite


class paris_aspa_reference(Variable):
    value_type = float
    entity = Famille
    definition_period = MONTH
    label = u"Minimum vieillesse à l'échelon national pour les personnes âgées"

    def formula(famille, period, parameters):

        personne_agee = famille.members('paris_personne_agee', period)
        personnes_agees_famille = famille.any(personne_agee)
        en_couple = famille('en_couple', period)
        aspa_montant_maximum_annuel = parameters(period).prestations_sociales.solidarite_insertion.minimum_vieillesse.aspa.montant_maximum_annuel

        
        return personnes_agees_famille * (en_couple * aspa_montant_maximum_annuel.couples + not_(en_couple) * aspa_montant_maximum_annuel.personnes_seules) / 12



# -*- coding: utf-8 -*-
from __future__ import division

from numpy import (maximum as max_, logical_not as not_, absolute as abs_, minimum as min_, select, where)

from openfisca_france.model.base import *  # noqa analysis:ignore


class paris_solidarite_ph_base_ressources(Variable):
    value_type = float
    entity = Famille
    definition_period = MONTH
    label = u"Base ressources mensuelles pour Paris Solidarité pour les personnes handicapées"
    reference = "article III.1.1.b.4 du règlement municipal du CASVP"

    def formula(famille, period, parameters):

        base_ressource = famille('paris_base_ressources_couple', period)
        
        aspa_montant_maximum_annuel = parameters(period).prestations_sociales.solidarite_insertion.minimum_vieillesse.aspa.montant_maximum_annuel
        aah = parameters(period).prestations_sociales.prestations_etat_de_sante.invalidite.aah
        en_couple = famille('en_couple', period)
        # ASPA est utilisé comme seuil de ressources pour
        # les couples avec personne en situation de handicap
        # faute d'AAH pour couple
        montant_psol_handicap = ((en_couple * aspa_montant_maximum_annuel.couples) / 12 + not_(en_couple) * aah.montant)

        return max_(montant_psol_handicap, base_ressource)


class paris_solidarite_ph_eligibilite(Variable):
    value_type = bool
    entity = Famille
    definition_period = MONTH
    label = u"Éligibilité à Paris Solidarité pour les personnes handicapées"
    
    def formula(famille, period, parameters):

        personnes_handicapees = famille.members('paris_personne_handicapee', period)
        return famille.any(personnes_handicapees, role = Famille.PARENT)


class paris_solidarite_ph_montant(Variable):
    value_type = float
    entity = Famille
    definition_period = MONTH
    label = u"Montant mensuel pour Paris Solidarité pour les personnes handicapées"
    reference = "article III.1.1.b.3 du règlement municipal du CASVP"

    def formula(famille, period, parameters):

        base_ressource = famille('paris_solidarite_ph_base_ressources', period)
        plafond_psol = parameters(period).paris.personnes_handicapees.psol.plafond

        en_couple = famille('en_couple', period)
        plafond = (en_couple * plafond_psol.couple + not_(en_couple) * plafond_psol.personne_seule)

        return max_(0, plafond - base_ressource)


class paris_solidarite_ph(Variable):
    value_type = float
    entity = Famille
    definition_period = MONTH
    label = u"Paris Solidarité pour les personnes handicapées"
    
    def formula(famille, period, parameters):

        montant = famille('paris_solidarite_ph_montant', period)
        ph_eligibilite = famille('paris_solidarite_ph_eligibilite', period)
        
        return ph_eligibilite * montant



# -*- coding: utf-8 -*-
from __future__ import division

from numpy import (maximum as max_, logical_not as not_, absolute as abs_, minimum as min_, select, where)

from openfisca_france.model.base import *  # noqa analysis:ignore


class paris_complement_sante(Variable):
    value_type = float
    label = u"Paris Complément Santé pour les personnes âgées et les personnes handicapées"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period, legislation):

        parisien = famille('parisien', period)

        pcs_pa = famille('paris_complement_sante_pa', period)
        pcs_ph = famille('paris_complement_sante_ph', period)

        return parisien * max_(pcs_pa, pcs_ph)



# -*- coding: utf-8 -*-
from __future__ import division

from numpy import (maximum as max_, logical_not as not_, absolute as abs_, minimum as min_, select, where)

from openfisca_france.model.base import *  # noqa analysis:ignore

# Paris Logement pour les personnes agées et les personnes handicapées

class paris_logement(Variable):
    value_type = float
    label = u"L'aide Paris Logement"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):
        paris_logement_pa_ph = famille('paris_logement_pa_ph', period)
        paris_logement_fam = famille('paris_logement_fam', period)
        paris_logement_apd = famille('paris_logement_apd', period)

        return paris_logement_pa_ph + paris_logement_fam + paris_logement_apd

class paris_logement_pa_ph(Variable):
    value_type = float
    label = u"Paris Logement pour les personnes handicapées et les personnes agées"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period, legislation):
        last_month = period.last_month

        plafond_pl = legislation(period).paris.paris_logement.plafond_pl
        plafond_pl_avec_enf = legislation(period).paris.paris_logement.plafond_pl_avec_enf
        aide_pers_isol = legislation(period).paris.paris_logement.aide_pers_isol
        aide_couple_ss_enf = legislation(period).paris.paris_logement.aide_couple_ss_enf
        aide_couple_avec_enf = legislation(period).paris.paris_logement.aide_couple_avec_enf

        base_ressources = famille('paris_base_ressources_foyer', last_month)
        loyer_net = famille('paris_logement_charge_nette_mensuelle', period)

        personnes_couple = famille('en_couple', period)
        nb_enfants = famille('paris_nb_enfants', period)
        paris_logement_ph_eligibilite = famille('paris_logement_ph_eligibilite', period)

        plafond = select([(nb_enfants >= 1), (nb_enfants < 1)], [plafond_pl_avec_enf, plafond_pl])
        condition_ressource = base_ressources <= plafond
        montant_aide = select([personnes_couple * (nb_enfants > 0), personnes_couple,
            ((personnes_couple != 1) * (nb_enfants == 0)), ((personnes_couple != 1) * (nb_enfants >= 1))],
            [aide_couple_avec_enf, aide_couple_ss_enf, aide_pers_isol, 0])

        result_montant = where((montant_aide > loyer_net), (montant_aide - (montant_aide - loyer_net)), montant_aide)

        result = where(result_montant > 0, result_montant, 0)

        paris_logement_pa = famille('paris_logement_pa', period)
        return max_(result * condition_ressource * paris_logement_ph_eligibilite, paris_logement_pa)


class paris_logement_pa_ph_eligibilite(Variable):
    value_type = bool
    label = u"Eligibilité à l'aide Paris Logement pour les personnes agées et les personne handicapées"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):
        parisien = famille('parisien', period)
        locataire = famille('paris_locataire', period)
        condition_taux_effort = famille('paris_condition_taux_effort', period)
        nb_enfants = famille('paris_nb_enfants', period)

        return parisien * locataire * condition_taux_effort * (nb_enfants <= 1)


class paris_logement_ph_eligibilite(Variable):
    value_type = bool
    label = u"Personne qui est eligible pour l'aide PL pour les personne handicapées"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):
        parisien = famille('parisien', period)
        locataire = famille('paris_locataire', period)
        condition_taux_effort = famille('paris_condition_taux_effort', period)

        personne_handicap_individu = famille.members('paris_personne_handicapee', period)
        personne_handicap = famille.sum(personne_handicap_individu)
        nb_enfants_handicapes = famille('paris_nb_enfants_handicapes', period)
        adulte_handicape = (personne_handicap - nb_enfants_handicapes) >= 1

        return parisien * locataire * condition_taux_effort * adulte_handicape

class paris_logement_fam(Variable):
    value_type = float
    label = u"Paris Logement pour les couples avec enfant(s)"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period, legislation):
        last_month = period.last_month

        plafond_pl_fam = legislation(period).paris.paris_logement.plafond_pl_fam
        aide_pl_fam = legislation(period).paris.paris_logement.aide_pl_fam

        base_ressources = famille('paris_base_ressources_foyer', last_month)
        loyer_net = famille('paris_logement_charge_nette_mensuelle', period)

        personnes_couple = famille('en_couple', period)
        nb_enfants = famille('paris_nb_enfants', period)
        paris_logement_elig_fam = famille('paris_logement_elig_fam', period)

        condition_ressource = base_ressources <= plafond_pl_fam

        montant_aide = where(personnes_couple * (nb_enfants > 0), aide_pl_fam, 0)

        result = where((montant_aide > loyer_net), (montant_aide - (montant_aide - loyer_net)), montant_aide)

        return result * condition_ressource * paris_logement_elig_fam

class paris_logement_elig_fam(Variable):
    value_type = bool
    label = u"Personne qui est eligible pour l'aide Paris Logement quand c'est un couple avec enfant(s)"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):
        parisien = famille('parisien', period)

        personnes_agees = famille.members('paris_personne_agee', period)
        personnes_agees_famille = famille.any(personnes_agees)

        personne_handicap = famille.members('paris_personne_handicapee', period)
        personne_handicap_famille = famille.any(personne_handicap)

        locataire = famille('paris_locataire', period)
        condition_taux_effort = famille('paris_condition_taux_effort', period)

        return parisien * locataire * (personnes_agees_famille != 1) * (personne_handicap_famille != 1) * condition_taux_effort

class paris_logement_apd(Variable):
    value_type = float
    label = u"Paris Logement pour les personnes isolées et couples sans enfant"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period, legislation):
        last_month = period.last_month

        plafond = legislation(period).paris.paris_logement.plafond_pl_apd
        aide_pl_apd_pers_isol = legislation(period).paris.paris_logement.aide_pl_apd_pers_isol
        aide_pl_apd_couple = legislation(period).paris.paris_logement.aide_pl_apd_couple

        base_ressources = famille('paris_base_ressources_foyer', last_month)

        indemnite = famille('paris_indemnite_enfant', last_month)
        loyer_net = famille('paris_logement_charge_nette_mensuelle', period)

        personnes_couple = famille('en_couple', period)
        paris_logement_elig_apd = famille('paris_logement_elig_apd', period)

        condition_ressources = (base_ressources - indemnite) <= plafond

        montant_aide = where(personnes_couple, aide_pl_apd_couple, aide_pl_apd_pers_isol)

        result = where((montant_aide > loyer_net), (montant_aide - (montant_aide - loyer_net)), montant_aide)

        return result * condition_ressources * paris_logement_elig_apd


class paris_logement_elig_apd(Variable):
    value_type = bool
    label = u"Personne qui est eligible pour l'aide Paris Logement aide aux parisiens en difficultés"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):
        parisien = famille('parisien', period)
        
        locataire = famille('paris_locataire', period)

        personnes_agees = famille.members('paris_personne_agee', period)
        personnes_agees_famille = famille.any(personnes_agees)

        personne_handicap = famille.members('paris_personne_handicapee', period)
        personne_handicap_famille = famille.any(personne_handicap)

        condition_taux_effort = famille('paris_condition_taux_effort', period)

        etudiant_ok = famille('paris_logement_elig_apd_etudiant', period)
        
        loyer = famille.demandeur.menage('loyer', period)

        nb_enfants = famille('paris_nb_enfants', period)

        return parisien * locataire * etudiant_ok * (personnes_agees_famille != 1) * (personne_handicap_famille != 1) * condition_taux_effort * (loyer > 0) * (nb_enfants == 0)


class paris_logement_elig_apd_etudiant(Variable):
    value_type = bool
    label = u"Évaluation de l'éligibilité des étudiants à Paris Logement aide aux parisiens en difficultés"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):
        etudiant_i = famille.members('etudiant', period)
        etudiant = famille.any(etudiant_i)

        boursier_i = famille.members('boursier', period)
        boursier = famille.any(boursier_i)

        return not_(etudiant) + (etudiant * boursier)


class paris_logement_charge_nette_mensuelle(Variable):
    value_type = float
    label = u"Charge nette de logement pour les locataires"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):

        loyer = famille.demandeur.menage('loyer', period)
        charges_locatives = famille.demandeur.menage('charges_locatives', period)
        aide_logement = famille('aide_logement', period)

        return max_(0, loyer + charges_locatives - aide_logement)



# -*- coding: utf-8 -*-
from openfisca_france.model.base import *  # noqa analysis:ignore


# Pass Paris Access est le nouveau nom du Navigo Emeraude Améthyste pour les personnes en situation de handicap
class paris_pass_access(Variable):
    value_type = bool
    label = u"Éligibilité au Pass Paris Access'"
    entity = Individu
    definition_period = MONTH

    def formula(individu, period):

        personnes_handicap = individu('paris_personne_handicapee', period)
        eligibilite_pass = individu('paris_pass_eligibilite', period)
        
        return personnes_handicap * eligibilite_pass



# -*- coding: utf-8 -*-
from __future__ import division

from numpy import absolute as abs_, minimum as min_, maximum as max_, where

from openfisca_france.model.base import *  # noqa analysis:ignore

class paris_logement_familles_elig(Variable):
    value_type = bool
    label = u"Eligibilité à Paris-Logement-Famille"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):

        parisien = famille('parisien', period)
        logement_a_charge = famille('paris_logement_a_charge', period)

        return parisien * logement_a_charge


class plf_handicap(Variable):
    value_type = float
    entity = Famille
    definition_period = MONTH
    label = u"Allocation Paris-Logement-Famille en cas d'enfant handicapé"

    def formula(famille, period, legislation):
        last_month = period.last_month

        br = famille('paris_base_ressources_foyer', last_month)
        P = legislation(period).paris.paris_logement_familles
        plafond_plfm = legislation(period).paris.plfm.deuxieme_plafond_plfm
        personnes_couple = famille('en_couple', period)

        parent_mono_plfm = (personnes_couple != 1) * (br > plafond_plfm)

        nb_enfant = famille('paris_nb_enfants', period)

        paris_enfant_handicape = famille.members('paris_enfant_handicape', period)
        nb_enf_handicape = famille.sum(paris_enfant_handicape)

        paris_enfant_handicape_garde_alternee = famille.members('paris_enfant_handicape_garde_alternee', period)
        nb_enf_handicape_garde_alternee = famille.sum(paris_enfant_handicape_garde_alternee)

        plafond = legislation(period).paris.paris_logement_familles.plafond_haut_3enf
        montant = legislation(period).paris.paris_logement_familles.montant_haut_3enf

        plf_handicap_elig = (nb_enf_handicape > 0) * (br <= plafond) * (personnes_couple + parent_mono_plfm)

        # Si tous les enfants handicapés sont en garde alternée
        deduction_garde_alternee = ((nb_enf_handicape - nb_enf_handicape_garde_alternee) == 0) * 0.5

        # S'il a plus de 3 enfants
        suppl_enfant = (br <= plafond) * (nb_enfant > 3) * P.montant_haut_enf_sup

        return plf_handicap_elig * ((1 - deduction_garde_alternee) * montant + suppl_enfant)

class paris_logement_familles(Variable):
    value_type = float
    label = u"Allocation Paris Logement Famille"
    entity = Famille
    definition_period = MONTH
    reference = "http://www.paris.fr/pratique/toutes-les-aides-et-allocations/aides-sociales/paris-logement-familles-prestation-ville-de-paris/rub_9737_stand_88805_port_24193"  # noqa

    def formula(famille, period, legislation):
        last_month = period.last_month

        elig = famille('paris_logement_familles_elig', period)
        br = famille('paris_base_ressources_foyer', last_month)
        personnes_couple = famille('en_couple', period)
        paris_enfant = famille.members('paris_enfant', period)
        nbenf = famille.sum(paris_enfant)
        paris_enfant_garde_alternee = famille.members('paris_enfant_garde_alternee', period)
        nbenf_garde_alternee = famille.sum(paris_enfant_garde_alternee)
        plf_handicap = famille('plf_handicap', period)
        menage = famille.demandeur.menage
        loyer = menage('loyer', period) + menage('charges_locatives', period)
        loyer_net = famille('paris_logement_charge_nette_mensuelle', period)
        P = legislation(period).paris.paris_logement_familles
        plafond_plfm = legislation(period).paris.plfm.deuxieme_plafond_plfm

        parent_mono_plfm = (personnes_couple != 1) * ((nbenf >= 4) + ((nbenf >= 2) * (br > plafond_plfm)))

        ressources_sous_plaf_bas = (br <= P.plafond_bas_3enf)
        ressources_sous_plaf_haut = (br <= P.plafond_haut_3enf) * (br > P.plafond_bas_3enf)

        montant_base_3enfs = (nbenf >= 3) * (
            ressources_sous_plaf_bas * P.montant_haut_3enf +
            ressources_sous_plaf_haut * P.montant_bas_3enf
            )
        montant_enf_sup = (
            ressources_sous_plaf_bas * P.montant_haut_enf_sup +
            ressources_sous_plaf_haut * P.montant_bas_enf_sup
            )
        montant_3enfs = (montant_base_3enfs + montant_enf_sup * max_(nbenf - 3, 0)) * (personnes_couple + parent_mono_plfm)
        montant_2enfs = ((nbenf == 2) * (br <= P.plafond_2enf) * P.montant_2enf) * (personnes_couple + parent_mono_plfm)
        plf = montant_2enfs + montant_3enfs
        deduction_garde_alternee = (nbenf_garde_alternee > 0) * (
            (nbenf - nbenf_garde_alternee < 3) * 0.5 * plf +
            (nbenf - nbenf_garde_alternee >= 3) * nbenf_garde_alternee * 0.5 * montant_enf_sup
            )

        plf = plf - deduction_garde_alternee
        plf = max_(plf, plf_handicap)
        plf = elig * plf
        plf = min_(plf, loyer)

        result_montant = where((plf > loyer_net), (plf - (plf - loyer_net)), plf)

        result = where(result_montant > 0, result_montant, 0)

        return result



# -*- coding: utf-8 -*-
from openfisca_france.model.base import *  # noqa analysis:ignore


class paris_personne_handicapee(Variable):
    value_type = bool
    label = u"Personne qui a le statut handicapé au sens de la Ville de Paris"
    entity = Individu
    definition_period = MONTH
    reference = "Article III.A.2 du Règlement municipal du CASVP"

    def formula(individu, period, legislation):

        handicap = individu('handicap', period)

        last_month = period.last_month
        aah = individu('aah', last_month)
        # manque : allocation compensatrice pour tierce personne
        pch = individu('pch', last_month)
        pensions_invalidite = individu('pensions_invalidite', last_month)
        # manque : pension de veuve ou de veuf invalide
        rente_accident_travail = individu('rente_accident_travail', last_month)
        # manque : pension anticipée attribuée aux fonctionnaires civils et aux militaires, s'ils ne sont pas admis à rester en service
        # manque : rente viagère d'invalidité servie par la Caisse des dépôts et consignations

        return handicap + aah + pch + pensions_invalidite + rente_accident_travail



# -*- coding: utf-8 -*-
from openfisca_france.model.base import *  # noqa analysis:ignore


class paris_pass_eligibilite_financiere(Variable):
    value_type = bool
    label = u"Éligibilité financière au Pass Paris Senior ou Pass Paris Access'"
    entity = FoyerFiscal
    definition_period = MONTH
    reference = "Articles II.3.1.b.1 et III.3.1.b.1 du règlement municipal du CASVP"

    def formula(foyer_fiscal, period, parameters):

        paris_pass = parameters(period).paris.paris_pass
        base_ressources = foyer_fiscal('ir_ss_qf', period.n_2)
        base_ressources_precedente = foyer_fiscal('ir_ss_qf', period.n_2.last_year)

        elig = base_ressources <= paris_pass.imposition_maximale
        elig_renouvellement = (base_ressources <= paris_pass.imposition_maximale_en_renouvellement) * (base_ressources_precedente <= paris_pass.imposition_maximale)

        return elig + elig_renouvellement


class paris_pass_eligibilite(Variable):
    value_type = bool
    label = u"Éligibilité au Pass Paris Senior ou Pass Paris Access'"
    entity = Individu
    definition_period = MONTH
    reference = "Articles II.3.1.b.1 et III.3.1.b.1 du règlement municipal du CASVP"

    def formula(individu, period):

        parisien = individu.famille('parisien', period)
        eligibilite_logement = individu.menage('statut_occupation_logement', period) != TypesStatutOccupationLogement.sans_domicile
        eligibilite_financiere = individu.foyer_fiscal('paris_pass_eligibilite_financiere', period)

        return parisien * eligibilite_logement * eligibilite_financiere




# -*- coding: utf-8 -*-
from __future__ import division

from numpy import (maximum as max_, logical_not as not_, absolute as abs_, minimum as min_, select)

from openfisca_france.model.base import *  # noqa analysis:ignore


class paris_forfait_familles_eligibilite(Variable):
    value_type = bool
    label = u"Eligibilité à l'aide Paris Forfait Familles"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):

        parisien = famille('parisien', period)
        logement_a_charge = famille('paris_logement_a_charge', period)
        nb_enfants = famille('paris_nb_enfants', period)

        return parisien * logement_a_charge * (nb_enfants >= 3)


class paris_forfait_familles_montant(Variable):
    value_type = float
    label = u"Montant de l'aide Paris Forfait Familles"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period, legislation):
        
        param = legislation(period).paris.familles.paris_forfait_familles
        base_ressources = famille('paris_base_ressources_couple', period.last_month)

        return select(
            [base_ressources <= param.plafond.premier, base_ressources <= param.plafond.deuxieme, base_ressources > param.plafond.deuxieme],
            [param.montant.premier_plafond, param.montant.deuxieme_plafond, 0]
            )


class paris_forfait_familles(Variable):
    value_type = float
    label = u"Aide Paris Forfait Familles"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period, legislation):

        eligibilite = famille('paris_forfait_familles_eligibilite', period)
        montant = famille('paris_forfait_familles_montant', period)

        return eligibilite * montant



# -*- coding: utf-8 -*-
from __future__ import division

from numpy import (maximum as max_, logical_not as not_, absolute as abs_, minimum as min_, select)

from openfisca_france.model.base import *  # noqa analysis:ignore

# Allocation de soutien aux parents d’enfants handicapés

class paris_logement_aspeh_base_ressources(Variable):
    value_type = float
    label = u"Base ressources pour l'Allocation de soutien aux parents d’enfants handicapés"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period, legislation):
        base_ressources = famille('paris_base_ressources_couple', period.last_month)
        clca = famille('paje_clca', period.last_month)
        return base_ressources + clca


class paris_logement_aspeh(Variable):
    value_type = float
    label = u"Le montant de l'Allocation de soutien aux parents d’enfants handicapés"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period, legislation):
        plafond_aspeh = legislation(period).paris.familles.aspeh.plafond_aspeh
        montant = legislation(period).paris.familles.aspeh.aide_aspeh

        parisien = famille('parisien', period)
        nb_enfant_handicape = famille('paris_nb_enfants_handicapes', period)
        ressources = famille('paris_logement_aspeh_base_ressources', period)

        return parisien * (ressources <= plafond_aspeh) * nb_enfant_handicape * montant



# -*- coding: utf-8 -*-
from openfisca_france.model.base import *  # noqa analysis:ignore


# Pass Paris Seniors est le nouveau nom du Navigo Emeraude Améthyste pour les personnes âgées
class paris_pass_seniors(Variable):
    value_type = bool
    label = u"Éligibilité au Pass Paris Seniors"
    entity = Individu
    definition_period = MONTH

    def formula(individu, period):

        personnes_agees = individu('paris_personne_agee', period)
        eligibilite_pass = individu('paris_pass_eligibilite', period)
        
        return personnes_agees * eligibilite_pass



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
    name="Openfisca-Paris",
    version="3.5.0",
    author="OpenFisca Team",
    author_email="contact@openfisca.fr",
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: POSIX",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Scientific/Engineering :: Information Analysis",
        ],
    description="Plugin OpenFisca pour les aides sociales de la mairie de Paris",
    keywords="benefit france paris microsimulation social tax",
    license="http://www.fsf.org/licensing/licenses/agpl-3.0.html",

    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'OpenFisca-Core >= 35.2.0, < 36',
        'OpenFisca-France >= 102, < 118'
    ],
    extras_require = {
        'test': [
            'nose',
            'pytest >= 5.4.2'
            ]
    },
)



# -*- coding: utf-8 -*-
from __future__ import division

from numpy import (maximum as max_, logical_not as not_, absolute as abs_, minimum as min_, select, where)

from openfisca_france.model.base import *  # noqa analysis:ignore


class paris_energie_familles_eligibilite_simple(Variable):
    value_type = bool
    label = u"Eligibilité à l'aide Paris Energie Familles, sans prendre en compte les conditions financières"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):
        parisien = famille('parisien', period)

        statut_occupation_logement = famille.demandeur.menage('statut_occupation_logement', period)
        participation_frais = famille.demandeur.menage('participation_frais', period)
        logement_a_charge = (
            famille('paris_logement_a_charge', period) +
            ((statut_occupation_logement == TypesStatutOccupationLogement.loge_gratuitement) * participation_frais)
            )

        return parisien * logement_a_charge


class paris_energie_familles_eligibilite_financiere(Variable):
    value_type = bool
    label = u"Eligibilité financière à l'aide Paris Energie Familles"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period, legislation):

        plafond = legislation(period).paris.familles.paris_energie_familles.plafond
        base_ressources = famille('paris_base_ressources_couple', period.last_month)

        nb_enfants = famille('paris_nb_enfants', period)
        nb_enfants_handicapes = famille('paris_nb_enfants_handicapes', period)

        return select(
            [(nb_enfants >= 3) + (nb_enfants_handicapes >= 1), nb_enfants == 2, nb_enfants == 1],
            [base_ressources <= plafond.trois_enfants, base_ressources <= plafond.deux_enfants, base_ressources <= plafond.un_enfant]
            )


class paris_energie_familles_montant(Variable):
    value_type = float
    label = u"Montant de l'aide Paris Energie Familles"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period, legislation):

        montant = legislation(period).paris.familles.paris_energie_familles.montant

        nb_enfants = famille('paris_nb_enfants', period)
        nb_enfants_handicapes = famille('paris_nb_enfants_handicapes', period)

        return select(
            [(nb_enfants >= 3) + (nb_enfants_handicapes >=1), (nb_enfants == 1) + (nb_enfants == 2)],
            [montant.trois_enfants, montant.un_ou_deux_enfants]
            )


class paris_energie_familles(Variable):
    value_type = float
    label = u"L'aide Paris Energie Familles"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period):

        eligibilite_simple = famille('paris_energie_familles_eligibilite_simple', period)
        eligibilite_financiere = famille('paris_energie_familles_eligibilite_financiere', period)
        montant = famille('paris_energie_familles_montant', period)

        return eligibilite_simple * eligibilite_financiere * montant



# -*- coding: utf-8 -*-
from __future__ import division

from numpy import (maximum as max_,  logical_not as not_,  absolute as abs_,  minimum as min_,  select, where)

from openfisca_france.model.base import *  # noqa analysis:ignore


class paris_logement_est_monoparentale(Variable):
    value_type = bool
    label = u"Famille monoparentale au sens de Paris logement familles monoparentales"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period, legislation):
        last_month = period.last_month

        parent_solo = not_(famille('en_couple', period))
        nb_enfants = famille('paris_nb_enfants', period)
        parisien = famille('parisien', period)

        logement_a_charge = famille('paris_logement_a_charge', period)

        return  parent_solo * (nb_enfants >= 1) * (nb_enfants < 4) * parisien * logement_a_charge


class paris_logement_plfm_montant(Variable):
    value_type = float
    label = u"Famille monoparentale qui est eligible à Paris logement familles monoparentales"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period, legislation):
        last_month = period.last_month

        premier_plafond_plfm = legislation(period).paris.plfm.premier_plafond_plfm
        deuxieme_plafond_plfm = legislation(period).paris.plfm.deuxieme_plafond_plfm
        aide_1er_plafond_plfm = legislation(period).paris.plfm.aide_1er_plafond_plfm
        aide_2eme_plafond_plfm = legislation(period).paris.plfm.aide_2eme_plafond_plfm

        base_ressources = famille('paris_base_ressources_foyer', last_month)

        return select([(base_ressources <= premier_plafond_plfm),
            (base_ressources <= deuxieme_plafond_plfm)], [aide_1er_plafond_plfm, aide_2eme_plafond_plfm])


class paris_logement_plfm(Variable):
    value_type = float
    label = u"Famille monoparentale qui est eligible à Paris logement familles monoparentales"
    entity = Famille
    definition_period = MONTH

    def formula(famille, period, legislation):
        last_month = period.last_month

        loyer_net = famille('paris_logement_charge_nette_mensuelle', period)
        est_monoparentale = famille('paris_logement_est_monoparentale', period)

        montant_aide_max = famille('paris_logement_plfm_montant', period)

        return est_monoparentale * min_(montant_aide_max, loyer_net)
